%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code to draw Fig. 6 in the main manuscript
%
% O. A. Elgendy and S. H. Chan,
% "Optimal Threshold Design for Quanta Image Sensor," 
% IEEE Transactions on Computational Imaging, 2017.
% http://ieeexplore.ieee.org/document/8169093/
% 
% Omar Elgendy and Stanley Chan
% Copyright 2017
% Purdue University, West Lafayette, In, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc,clear,close all
K                =  8;
theta            =  4.5;
Y                =  poissrnd(theta*ones(K));
gamma1           =  0.5;
oracleq          =  floor(theta)+1;
NbBisectionIter  =  8;
qmax             =  16;
Biter            =  {'k',[1 1 0]};
for q=0:qmax
    B            =  1*(Y>=q);
    v(q+1,1)     =  mean(B(:));
end
set(groot, 'defaultAxesTickLabelInterpreter','latex'); set(groot, 'defaultLegendInterpreter','latex');
figure('Color',[1 1 1],'Position',[7 -223 1108 908])
plot(0:qmax,v,'-bo','linewidth',3);hold on;
xlabel('Threshold $q$');
ylabel('Bit density $1-\gamma_q(c)$');
set(gca,'fontsize',20,'fontweight','bold','LineWidth',2)

B                =  1*(Y>=oracleq);
oraclev          =  mean(B(:));


c1 = [0.75 0.25 0.25];
hold on;line([oracleq oracleq],[0 1],'linestyle','--','linewidth',2,'Color',c1);
hold on;line([0 qmax],[oraclev oraclev],'linestyle','--','linewidth',2,'Color',c1);

%------------------------
qlow             =  0;
B                =  1*(Y>=qlow);
vlow             =  mean(B(:));
plot(qlow,vlow,'o','LineWidth',2,'markersize',12,...
    'MarkerEdgeColor','k')%,...
    %'MarkerFaceColor',[1,1,0])
%------------------------
qhigh            =  qmax;
B                =  1*(Y>=qhigh);
vhigh            =  mean(B(:));
plot(qhigh,vhigh,'o','LineWidth',2,'markersize',12,...
    'MarkerEdgeColor',Biter{1})%,...
    %'MarkerFaceColor',Biter{2})
%------------------------
qcenter          =  round((qlow+qhigh)/2);
B                =  1*(Y>=qcenter);
b(:,:,1)         =  B;
qiter(1)         =  qcenter;
vcenter          =  mean(B(:));
plot(qcenter,vcenter,'o','LineWidth',2,'markersize',12,...
    'MarkerEdgeColor',Biter{1})%,...
    %'MarkerFaceColor',Biter{2})
    xlim([0 qmax])
%------------------------

t2=tic;
errQ             =  zeros(NbBisectionIter+1-3,1);
gamma_iter       =  zeros(NbBisectionIter+1-3,1);
errQ(1)          =  (qcenter-oracleq)^2;
gamma_iter(1)    =  vcenter;
for i=1:NbBisectionIter-3
    if vcenter  >=  gamma1
        qlow     =  qcenter;
        vlow     =  vcenter;
    else
        qhigh    =  qcenter;
        vhigh    =  vcenter;
    end
    qcenter            =  round((qlow+qhigh)/2);
    B                  =  1*(Y>=qcenter);
    b(:,:,i+1)         =  B;
    vcenter            =  mean(B(:));
    qiter(i+1)         =  qcenter;
    errQ(i+1,1)        =  (qcenter-oracleq)^2;
    gamma_iter(i+1,1)  =  vcenter;
    plot(qcenter,vcenter,'o','LineWidth',2,'markersize',12,...
    'MarkerEdgeColor',Biter{1})%,...
    %'MarkerFaceColor',Biter{2})
    fprintf('qcenter = %d\n',qcenter);
end
qBisection             =  min(max(qcenter,1),qmax);

figure;
%axes('Position',[.5 .24 .2 .2])
imagesc(b(:,:,1));colormap(gray(256));box on;grid on;hold on;
set(gca,'xtick',[],'ytick',[],'LineWidth',2,'XColor',[0 0 0],'YColor',[0 0 0])
for i=0.5:8.5
    line([-1 8.5],[i i],'LineWidth',2,'Color',[0 0 0])
    line([i i],[-1 8.5],'LineWidth',2,'Color',[0 0 0])
end
figure;
%axes('Position',[.4 .7 .2 .2])
imagesc(b(:,:,2));colormap(gray(256));box on;grid on;
set(gca,'xtick',[],'ytick',[],'LineWidth',2,'XColor',[0 0 0],'YColor',[0 0 0])
hold on;
for i=0.5:8.5
    line([-1 8.5],[i i],'LineWidth',2,'Color',[0 0 0])
    line([i i],[-1 8.5],'LineWidth',2,'Color',[0 0 0])
end
figure;
%axes('Position',[.15 .15 .2 .2])
imagesc(b(:,:,3));colormap(gray(256));box on;grid on;
set(gca,'xtick',[],'ytick',[],'LineWidth',2,'XColor',[0 0 0],'YColor',[0 0 0])
hold on;
for i=0.5:8.5
    line([-1 8.5],[i i],'LineWidth',2,'Color',[0 0 0])
    line([i i],[-1 8.5],'LineWidth',2,'Color',[0 0 0])
end
figure;
%axes('Position',[.45 .48 .2 .2])
imagesc(b(:,:,4));colormap(gray(256));box on;grid on;hold on;
set(gca,'xtick',[],'ytick',[],'LineWidth',2,'XColor',[0 0 0],'YColor',[0 0 0])
for i=0.5:8.5
    line([-1 8.5],[i i],'LineWidth',2,'Color',[0 0 0])
    line([i i],[-1 8.5],'LineWidth',2,'Color',[0 0 0])
end
%%
x=im2double(imread('C:\Temp\oelgendy\Work\images\IMG_01862.jpg'));
[rows,cols,~]=size(x);
figure;imshow(x);hold on
for i=1:70:max(rows,cols)
    line([0 cols],[i i],'LineWidth',2,'Color',[0 0 0])
    line([i i],[0 rows],'LineWidth',2,'Color',[0 0 0])
end
line([211 281],[281 281],'LineWidth',2,'Color',[1 0 0])
line([211 281],[351 351],'LineWidth',2,'Color',[1 0 0])
line([211 211],[281 351],'LineWidth',2,'Color',[1 0 0])
line([281 281],[281 351],'LineWidth',2,'Color',[1 0 0])