%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bisection Algorithm for Threshold Adaptation
%
% O. A. Elgendy and S. H. Chan
% "Image Reconstruction and Threshold Design for Quanta Image Sensor", 
% IEEE Transactions on Computational Imaging, 2017.
% http://ieeexplore.ieee.org/abstract/document/7532503/
% 
% Omar Elgendy and Stanley Chan
% Copyright 2017
% Purdue University, West Lafayette, In, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Q_map,t_bisection,gamma_1f,B,errQ,Q_map_iter,gamma_iter]=BisectionAdapt(y0,qmax,Qblocksize,oracleQmap,NbBisectionIter,Pixidx,flag)
gamma1                 =  0.5;
[Urows,Ucols,T]        =  size(y0);
[rows,~]               =  size(oracleQmap);
K                      =  round(Urows/rows);
g                      =  ones(K);
Total_block_size       =  Qblocksize*K;
NbBisectionIter        =  min(NbBisectionIter,T);
Qmap_size              =  floor([Urows,Ucols]/Total_block_size);
ActualSize             =  Qmap_size*Qblocksize*K;
g_Qmap                 =  ones(Qblocksize);
%------------------------
qlow                   =  zeros(Qmap_size);
B                      =  1*(y0(1:ActualSize(1),1:ActualSize(2),1)>=kron(qlow,kron(g_Qmap,g)));
vlow                   =  blockfun(B(1:ActualSize(1),1:ActualSize(2)),Total_block_size*[1 1],@mean)-gamma1;
%------------------------
qhigh                  =  qmax*ones(Qmap_size);
B                      =  1*(y0(1:ActualSize(1),1:ActualSize(2),2)>=kron(qhigh,kron(g_Qmap,g)));
vhigh                  =  blockfun(B(1:ActualSize(1),1:ActualSize(2)),Total_block_size*[1 1],@mean)-gamma1;
%------------------------
qcenter                =  ceil((qlow+qhigh)/2);
B                      =  1*(y0(1:ActualSize(1),1:ActualSize(2),3)>=kron(qcenter,kron(g_Qmap,g)));
vcenter                =  blockfun(B(1:ActualSize(1),1:ActualSize(2)),Total_block_size*[1 1],@mean)-gamma1;
%------------------------
t2=tic;
errQ                   =  zeros(NbBisectionIter+1-3,1);
if flag==1
    errQ(1)            =  norm(qcenter(:)-oracleQmap(:),2)^2/numel(oracleQmap);
end
Q_map_iter(:,:,1)      =  [qlow(:) qcenter(:) qhigh(:)];
gamma_iter(:,1)        =  vcenter(:);
for i=1:NbBisectionIter-3
    qlow(vcenter>=0)   =  qcenter(vcenter>=0);
    vlow(vcenter>=0)   =  vcenter(vcenter>=0);
    qhigh(vcenter<0)   =  qcenter(vcenter<0);
    vhigh(vcenter<0)   =  vcenter(vcenter<0);
    qcenter            =  ceil((qlow+qhigh)/2);
    B                  =  1*(y0(1:ActualSize(1),1:ActualSize(2),i+3)>=kron(qcenter,kron(g_Qmap,g)));
    vcenter            =  blockfun(B(1:ActualSize(1),1:ActualSize(2)),Total_block_size*[1 1],@mean)-gamma1;
    if flag==1
        errQ(i+1,1)    =  norm(qcenter(:)-oracleQmap(:),2)^2/numel(qcenter);
    end
    Q_map_iter(:,:,i+1)=  [qlow(:) qcenter(:) qhigh(:)];
    gamma_iter(:,i+1)  =  vcenter(:);
    %fprintf('||va-vc||= %3.2f \t ||vb-vc|| = %3.2f \n', norm(vleft-vcenter),norm(vright-vcenter));
end
t_bisection            =  toc(t2);
Q_map                  =  kron(qcenter,kron(g_Qmap,g));
Q_map                  =  [Q_map;repmat(Q_map(end,:),max(0,size(y0,1)-ActualSize(1)),1)]; % repeat last row to have dim of y0
Q_map                  =  [Q_map repmat(Q_map(:,end),1,max(0,size(y0,2)-ActualSize(2)))]; % repeat last col to have dim of y0
Q_map                  =  min(max(Q_map,1),qmax);
Q_map_iter             =  Q_map_iter(Pixidx,:,:);
gamma_1f               =  vcenter+gamma1;
