%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implementation of the Reconstruction Algorithm according to 
% [1] T. Vogelsang, D. G. Stork, and M. Guidash, "Hardware validated unified
% model of multibit temporally and spatially oversampled image sensors with
% conditional reset," J. Electronic Imaging, vol. 23, no. 1, pp. 013021, 2014.
%
% O. A. Elgendy and S. H. Chan
% "Image Reconstruction and Threshold Design for Quanta Image Sensor", 
% IEEE Transactions on Computational Imaging, 2017.
% http://ieeexplore.ieee.org/abstract/document/7532503/
% 
% Omar Elgendy and Stanley Chan
% Copyright 2017
% Purdue University, West Lafayette, In, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [IM_CR,PSNR_CR]=ConditionalResetAlg(y0n,Kt,Q_sequence,c0)

[rows,cols]               =  size(c0);
marks                     =  ceil(logspace(log10(1),log10(Kt),length(Q_sequence)+1));

q                         =  zeros(Kt,1);
for i=1:length(marks)-1
    q(marks(i):marks(i+1))=  Q_sequence(i);
end

Y                         =  zeros(rows,cols);
for t = 1:Kt-1
    B                     =  1*(y0n(:,:,t)>=q(t));
    Y                     =  Y + B;
    y0n(:,:,t+1)          =  y0n(:,:,t+1) + y0n(:,:,t).*(1-B);    % accumlate the unresetted pixels to the next frame
end
IM_CR                     =  Y/max(Y(:));
PSNR_CR                   =  psnr(IM_CR,c0);