%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implementation of the Markov Chain Algorithm for Threshold Adaptation
% according to 
% [1] C. Hu and Y. M. Lu, "Adaptive time-sequential binary sensing for high
% dynamic range imaging," in Proc. SPIE Conf. Adv. Photon Counting,
% 2012, vol. 8375, pp. 83750A1.
%
% O. A. Elgendy and S. H. Chan
% "Image Reconstruction and Threshold Design for Quanta Image Sensor", 
% IEEE Transactions on Computational Imaging, 2017.
% http://ieeexplore.ieee.org/abstract/document/7532503/
% 
% Omar Elgendy and Stanley Chan
% Copyright 2017
% Purdue University, West Lafayette, In, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Q_map,b,t_Markov,err_Qmap,Q_map_iter]=MCAdapt(y,qmax,oracleQmap,L,Beta,Pixidx)
%rng('default')
%qmax=qmax-1;
addpath(genpath('gDiscrPdfRnd_funct/'));
%L                  =  2;       %number of bits to represent 1 threshold
%Beta               =  0.1;     %staying Probability
Alpha              =  1-Beta;
NumStates          =  2^L*qmax; %number of states in Markov Chain
[rowsU,colsU,Kt]   =  size(y);  % K here is just temporal oversampling
[rows,cols]        =  size(oracleQmap);
Ks                 =  round(rowsU/rows);
%---------------indices for raster scan--------
%[repmat(x1,1,4)' kron(x1,ones(1,4))']
s=1;
idx=zeros(rows*cols,Ks^2);
for s1=1:Ks
    for s2=1:Ks
        rowsidx=s1:Ks:rowsU;
        colsidx=s2:Ks:colsU;
        pixidx=[repmat(rowsidx,1,length(colsidx))' kron(colsidx,ones(1,length(rowsidx)))'];% blocks are indexed in MATLAB order cols
        %pixidx=[kron(rowsidx,ones(1,length(colsidx)))' repmat(colsidx,1,length(rowsidx))' ];% blocks are indexed in raster order cols
        idx(:,s)=sub2ind([rowsU,colsU], pixidx(:,1),pixidx(:,2)); % indices of element (s1,s2) in every block in raster order,
        s=s+1;
    end
end
%------------Adaptive Threshold Update---------
%oldState          =  2^L*qmax*ones(rowsU*K,colsU*K);%we start the chain from the rightmost state, corresponding to an initial threshold q0 = 10
%oldState          =  randi(NumStates,rows,cols);
%oldQ_map          =  floor((oldState-1)/2^L)+1;
oldQ_map           =  floor(qmax/2)*ones(rows,cols);
oldState           =  NumStates/2*ones(rows,cols);
b                  =  1*(y(:,:,1)>=kron(oldQ_map,ones(Ks)));
err_Qmap           =  zeros(Kt,1);
err_Qmap(1)        =  norm(oldQ_map(:)-oracleQmap(:),2)^2/numel(oldQ_map);
Q_map_iter(:,1)    =  oldQ_map(:);
t2=tic;
% figure;imagesc(oldQ_map)

for t=1:Kt-1
%     fprintf('Frame %d\n',t)
    for s=1:Ks^2
        R                     =  binornd(1,Alpha,[rows cols]); %bernoulli r.v. to specify wether to stay in current state or move to other state
        Deltam                =  zeros(rows,cols);
        smallb                =  reshape(b(idx(:,s)),rows,cols); % check b or b'????????
        Deltam(R==1 & smallb==0)   =  -1;
        Deltam(R==1 & smallb==1)   =  1;
        newState              =  max(min(oldState+Deltam,NumStates),1);
        newQ_map              =  max(min(floor((newState-1)/2^L)+1,qmax),1);
        oldState              =  newState;
%         imagesc(newQ_map);drawnow
    end
    b                         =  1*(y(:,:,t+1)>=kron(newQ_map,ones(Ks)));
    err_Qmap(t+1,1)           =  norm(newQ_map(:)-oracleQmap(:),2)^2/numel(newQ_map);
    Q_map_iter(:,t+1)         =  newQ_map(:);
end
t_Markov=toc(t2);
Q_map_iter =  Q_map_iter(Pixidx,:);
Q_map=newQ_map;
