%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Demo of Threshold Adaptation and Image Reconstruction
%
% O. A. Elgendy and S. H. Chan,
% "Optimal Threshold Design for Quanta Image Sensor,"
% IEEE Transactions on Computational Imaging, 2017.
% http://ieeexplore.ieee.org/document/8169093/
%
% Omar Elgendy and Stanley Chan
% Copyright 2017
% Purdue University, West Lafayette, In, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc,clear,close all
rng('default');
addpath(genpath('./ImageData/'));
addpath(genpath('./Utilities'));

%% QIS Simulation Parameters
K                =  4;                % Spatial  Overasampling Factor
T_adapt          =  8;			      % Frames for Threshold Adaptation
T_recon          =  22;               % Frames for Image Recosntruction
T                =  T_adapt+T_recon;  % Temporal Overasampling Factor
Qsize            =  K;                % Kernel size of pixels that share the same threshold
qmax             =  16;               % Maximum Threshold
g                =  ones(K)/K^2;      % Box-car Interpolation kernel

%% Read an image, and simulate photon counts
IM = imresize(im2double(imread('./ImageData/SSIPDataset/IMG_0138.JPG')),0.1);
if size(IM,3)~=1;
    IM           =  rgb2gray(IM);
end
[rows,cols]      =  size(IM);
alpha            =  K^2*(qmax-1);    % Sensor gain
oracleQmap       =  floor(alpha*IM/K^2)+1;
y                =  generatePhotonCounts(IM,K,T,alpha,g);

%% Generate the adapted threshold map
opts.y           =  y;
opts.qmax        =  qmax;
opts.oracleQmap  =  oracleQmap;
opts.T_adapt     =  T_adapt;
opts.Qblock      =  Qsize;
[QmapLR,QmapHR]  =  generateQmap([rows,cols],K,'bisection',opts);

%% Generate the binary measurements
reconFrames      =  T_adapt + (1:T_recon);
b                =  1*(y(:,:,reconFrames)>=repmat(QmapHR,1,1,T_recon));

%% Reconstruct the image
IM_qs            =  imageReconstruct(b,K,alpha,QmapLR);
PSNR             =  psnr(IM_qs,IM);

%% Draw the Reconstructed iamge and Ground Truth
figure;
subplot(121);imshow(IM);title('Ground Truth')
subplot(122);imshow(IM_qs);title(sprintf('PSNR=%2.2f',PSNR));